/**
 *
 */
package jp.ac.nao.jvo.research.clustering;

import java.util.List;
import java.util.ArrayList;

//import jp.ac.nao.jvo.infra.log.JVOLogger;

/**
 *
 */
public class CountMap {

    private static int MIN_COUNT = 10;
    private static double MIN_SIZE = 100 / 60. / 60. / 180. * Math.PI;    

    private String id;
    private int count;
    private double xmin;
    private double xmax;
    private double ymin;
    private double ymax;

    private boolean dead = false;

    private CountMap[] map = null;

    /**
     *
     */
    public CountMap(String id,
                    double xmin, double xmax, double ymin, double ymax, 
                    List<double[]> posList, boolean recursive) {

        //JVOLogger.trace(id + " " + posList.size() + " " + recursive);
        this.id = id;

        this.xmin = xmin;
        this.xmax = xmax;
        this.ymin = ymin;
        this.ymax = ymax;

        double xsize = xmax - xmin;
        double ysize = ymax - ymin;
        boolean stop = false;
        if(xsize < MIN_SIZE && ysize < MIN_SIZE) {
            stop = true;
        }

        if(posList != null) {
            this.count = posList.size();
            if(this.count > 0 && recursive && stop == false) {
                divide(posList);
            }
        } else {
            this.count = 0;
        }
    }
    
    /**
     *
     */
    public int getCount() {
        return this.count;
    }
    
    /**
     *
     */
    public int getCount(int index) {
        if(index >= 0 && index < 4) {
            CountMap map = this.map[index];
            if(map == null) {
                return 0;
            } else {
                return map.getCount();
            }
        } else {
            return 0;
        }
    }

    /**
     *
     */
    public void setDead(boolean b) {
        this.dead = b;
    }

    /**
     *
     */
    public boolean isDead() {
        return this.dead;
    }

    /**
     *
     */
    public boolean isNext(CountMap map) {
        List<double[]> list0 = getCorner();
        List<double[]> list1 = map.getCorner();
        int count = 0;

        for(int i = 0; i < list0.size(); i++) {
            int k = i + 1;
            if(k >= list0.size()) {
                k = 0;
            }
            double xa = list0.get(i)[0];
            double ya = list0.get(i)[1];
            double xb = list0.get(k)[0];
            double yb = list0.get(k)[1];
            
            double a = 0;
            double b = 0;
            boolean flag = true;

            if(xa > xb || xa < xb) {
                a = (ya - yb) / (xa - xb);
                b = ya - a * xa;
            } else {
                flag = false;
            }

            for(int j = 0; j < list1.size(); j++) {
                double x1 = list1.get(j)[0];
                double y1 = list1.get(j)[1];

                if(flag) {
                    double tmp = (a * x1 - y1 + b) / Math.sqrt(a * a + 1);
                    double dist = Math.abs(tmp);
                    if(dist < MIN_SIZE * 0.1) {
                        if(xa < xb) {
                            if(x1 >= xa && x1 <= xb) {
                                count++;
                            }
                        } else {
                            if(x1 >= xb && x1 <= xa) {
                                count++;
                            }
                        }
                    }
                } else {
                    double dist = Math.abs(x1 - xa);
                    if(dist < MIN_SIZE * 0.1) {
                        if(ya < yb) {
                            if(y1 >= ya && y1 <= yb) {
                                count++;
                            }
                        } else {
                            if(y1 >= yb && y1 <= ya) {
                                count++;
                            }
                        }
                    }
                }
                if(count >= 2) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     *
     */
    public List<double[]> getCorner() {
        List<double[]> list =  new ArrayList<double[]>();
        double[] pos0 = new double[2];
        double[] pos1 = new double[2];
        double[] pos2 = new double[2];
        double[] pos3 = new double[2];
        pos0[0] = this.xmax;
        pos0[1] = this.ymax;
        pos1[0] = this.xmin;
        pos1[1] = this.ymax;
        pos2[0] = this.xmin;
        pos2[1] = this.ymin;
        pos3[0] = this.xmax;
        pos3[1] = this.ymin;
        list.add(pos0);
        list.add(pos1);
        list.add(pos2);
        list.add(pos3);
        return list;
    }

    
    /**
     *
     */
    public double getArea() {
        return (xmax - xmin) * (ymax - ymin);
    }


    /**
     *
     */
    public CountMap getCountMap(int index) {
        if(index >= 0 && index < 4) {
            return this.map[index];
        } else {
            return null;
        }
    }
    
    /**
     *
     */
    public List<CountMap> getZeroCountMap() {
        List<CountMap> list = new ArrayList<CountMap>();
        if(this.count == 0) {
            list.add(this);
        } else {
            if(this.map != null) {
                for(int i = 0; i < 4; i++) {
                    if(this.map[i] != null) {
                        list.addAll(this.map[i].getZeroCountMap());
                    }
                }
            }
        }
        return list;
    }

    /**
     *
     */
    public List<CountMap> getNonZeroCountMap() {
        List<CountMap> list = new ArrayList<CountMap>();

        if(this.count == 0) {
            return list;
        } else {
            if(this.map == null) {
                list.add(this);
            } else {
                for(int i = 0; i < 4; i++) {
                    if(this.map[i] != null) {
                        list.addAll(this.map[i].getNonZeroCountMap());
                    }
                }
            }
        }
        return list;
    }

    /**
     *
     */
    private void divide(List<double[]> posList) {
        double xmid = (this.xmax + this.xmin) * 0.5;
        double ymid = (this.ymax + this.ymin) * 0.5;

        List<double[]> list0 = new ArrayList<double[]>();
        List<double[]> list1 = new ArrayList<double[]>();
        List<double[]> list2 = new ArrayList<double[]>();
        List<double[]> list3 = new ArrayList<double[]>();
        
        String id0 = this.id + "0";
        String id1 = this.id + "1";
        String id2 = this.id + "2";
        String id3 = this.id + "3";

        for(int i = 0; i < posList.size(); i++) {
            double[] pos = posList.get(i);
            if(pos[0] >= xmid) {
                if(pos[1] >= ymid) {
                    list0.add(pos);
                } else {
                    list3.add(pos);
                }
            } else {
                if(pos[1] >= ymid) {
                    list1.add(pos);
                } else {
                    list2.add(pos);
                }
            }
        }

        boolean b = true;
        if(getCount() < MIN_COUNT) {
            b = false;
            int count = 0;
            if(list0.size() == 0) {
                count++;
            }
            if(list1.size() == 0) {
                count++;
            }
            if(list2.size() == 0) {
                count++;
            }
            if(list3.size() == 0) {
                count++;
            }
            if(count == 2 || count == 3) {
                b = true;
            }
        }
        this.map = new CountMap[4];
        this.map[0] = new CountMap(id0, xmid, xmax, ymid, ymax, list0, b);
        this.map[1] = new CountMap(id1, xmin, xmid, ymid, ymax, list1, b);
        this.map[2] = new CountMap(id2, xmin, xmid, ymin, ymid, list2, b);
        this.map[3] = new CountMap(id3, xmid, xmax, ymin, ymid, list3, b);
    }

    /**
     *
     */
    public boolean contains(double[] pos) {
        if(pos[0] >= xmin && pos[0] < xmax &&
           pos[1] >= ymin && pos[1] < ymax) {
            return true;
        } else {
            return false;
        }
    }

    /**
     *
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        //sb.append(id).append(" ").append(count).append(" ")
        sb
            .append(xmin).append(" ")
            .append(xmax).append(" ")
            .append(ymin).append(" ")
            .append(ymax);
        return sb.toString();
    }


}

